/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	calentool.c	read and write calentool appointment files	*/
/*									*/
/************************************************************************/

#include	<stdio.h>
#include	<ctype.h>

#include	"manifest.h"
#include	"calentool.h"

PRIVATE	char	*days[] = {"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa", "**"};

/************************************************************************/
PRIVATE	int	scan_value(p, val)

char	**p;
int	*val;

{	int	i;

	if (*(*p + 2) != ' ')
	   return(FALSE);
	else if (isdigit(**p) && isdigit(*(*p + 1))) {
	   *val = (**p - '0') * 10 + (*(*p + 1)) - '0';
	   *p += 3;
	   return(TRUE);
	   }
	else {
	   for (i = 0; i < 8; i++)
	      if (strncmp(*p, days[i], 2) == 0) {
	         *val = -(i + 1);
	         *p += 3;
	         return(TRUE);
	         }
	   return(FALSE);
	   }
}

/************************************************************************/
PRIVATE	print_value(f, val)

FILE	*f;
int	val;

{
	if (val < 0)
	   fprintf(f, "%s ", days[-val - 1]);
	else
	   fprintf(f, "%02d ", val);
}

/************************************************************************/
PRIVATE	free_calentool(cal)

calen	*cal;

{	calen	*old;

	while (cal) {
	   old = cal;
	   cal = cal->next;
	   if (old->message)
	      free(old->message);
	   if (old->warning)
	      free(old->warning);
	   if (old->interval)
	      free(old->interval);
	   if (old->repeat)
	      free(old->repeat);
	   free(old);
	   }
}

/************************************************************************/
EXPORT	calen	*load_calentool(path)

char	*path;

{	calen	*head = NULL, *cal;
	FILE	*f;
	char	*p, *q, buf[1024], *index();

	if ((f = fopen(path, "r")) != NULL) {
	   while (fgets(buf, 1024, f)) {
	      if (head == NULL)
	         head = cal = (calen *) malloc(sizeof(calen));
	      else {
	         cal->next = (calen *) malloc(sizeof(calen));
	         cal = cal->next;
	         }
	      bzero(cal, sizeof(calen));
	      buf[strlen(buf) - 1] = '\0';
	      for (p = buf; isspace(*p); p++)
	         ;
	      if (*p == '\0' || *p == '\n' || *p == '\r') {
	         cal->kind = LINE_BLANK;
	         }
	      else if (buf[0] == '#') {
	         cal->kind = LINE_COMMENT;
	         cal->message = strsave(buf + 1);
	         }
	      else {
	         cal->kind = LINE_APPOINTMENT;
	         p = buf;
	         if (!scan_value(&p, &(cal->year))) {
	            error("Invalid year in appointment file");
	            free_calentool(head);
	            return(NULL);
	            }
	         if (!scan_value(&p, &(cal->month))) {
	            error("Invalid month in appointment file");
	            free_calentool(head);
	            return(NULL);
	            }
	         if (!scan_value(&p, &(cal->day))) {
	            error("Invalid day in appointment file");
	            free_calentool(head);
	            return(NULL);
	            }
	         if (!scan_value(&p, &(cal->hour))) {
	            error("Invalid hour in appointment file");
	            free_calentool(head);
	            return(NULL);
	            }
	         if (!scan_value(&p, &(cal->minute))) {
	            error("Invalid minute in appointment file");
	            free_calentool(head);
	            return(NULL);
	            }
	         if (!scan_value(&p, &(cal->length))) {
	            error("Invalid length in appointment file");
	            free_calentool(head);
	            return(NULL);
	            }
	         while (*p && isspace(*p))
	            p++;
	         if (*p == '[') {
	            if ((q = index(p, ']')) == NULL) {
	               error("Invalid interval in appointment file");
	               free_calentool(head);
	               return(NULL);
	               }
	            cal->interval = (char *) malloc(q - p + 2);
	            strncpy(cal->interval, p, q - p + 1);
	            *(cal->interval + (q - p) + 1) = '\0';
	            p = q + 1;
	            }
	         while (*p && isspace(*p))
	            p++;
	         if (*p == '<') {
	            if ((q = index(p, '>')) == NULL) {
	               error("Invalid warning in appointment file");
	               free_calentool(head);
	               return(NULL);
	               }
	            cal->warning = (char *) malloc(q - p + 2);
	            strncpy(cal->warning, p, q - p + 1);
	            *(cal->warning + (q - p) + 1) = '\0';
	            p = q + 1;
	            }
	         while (*p && isspace(*p))
	            p++;
	         if (*p == '+') {
	            for (p++, cal->repeat = 0; isdigit(*p); p++)
	               cal->repeat = cal->repeat * 10 + *p - '0';
	            }
	         else
	            cal->repeat = -1;
	         while (*p && isspace(*p))
	            p++;
	         if (*p != '\0')
	            cal->message = strsave(p);
	         }
	      }
	   fclose(f);
	   return(head);
	   }
	else {
	   error("Cannot read ", path);
	   return(NULL);
	   }
}

/************************************************************************/
EXPORT	store_calentool(path, list)

char	*path;
calen	*list;

{	FILE	*f;
	calen	*cal;

	if ((f = fopen(path, "w")) != NULL) {
	   for (cal = list; cal; cal = cal->next)
	      if (cal->kind == LINE_BLANK)
	         fprintf(f, "\n");
	      else if (cal->kind == LINE_COMMENT)
	         fprintf(f, "#%s\n", cal->message);
	      else {
	         print_value(f, cal->year);
	         print_value(f, cal->month);
	         print_value(f, cal->day);
	         print_value(f, cal->hour);
	         print_value(f, cal->minute);
	         print_value(f, cal->length);
	         if (cal->interval)
	            fprintf(f, "%s ", cal->interval);
	         if (cal->warning)
	            fprintf(f, "%s ", cal->warning);
	         if (cal->repeat != -1)
	            fprintf(f, "+%d ", cal->repeat);
	         if (cal->message)
	            fprintf(f, cal->message);
	         fprintf(f, "\n");
	         }
	   fclose(f);
	   }
	else
	   error("Cannot write to ", path);
	free_calentool(list);
}
