/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	casio_master.c	send i/o requests to Casio slave		*/
/*									*/
/************************************************************************/

#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/termios.h>
#include	<sys/file.h>

#include	"manifest.h"
#include	"protocol.h"

PUBLIC	int	errno;
PUBLIC	char	*sys_errlist[];

PRIVATE	FILE	*commands_out;
PRIVATE	FILE	*results_in;
PRIVATE	char	buf[512];

/************************************************************************/
PRIVATE	int	do_write(buf, len)

char	*buf;
int	len;

{
	if (fwrite(buf, 1, len, commands_out) == len)
	   if (fflush(commands_out) == 0)
	      return(TRUE);
	   else
	      error("Could not flush command pipe to Casio slave");
	else
	   error("Error writing to Casio slave command pipe: %s", sys_errlist[errno]);
	return(FALSE);
}

/************************************************************************/
PRIVATE	int	read_reply()

{	char	result;

	if (fread(&result, 1, 1, results_in) == 1)
	   if (result == 0)
	      return(TRUE);
	   else if (fread(buf, 1, result, results_in) == result)
	      error(buf);
	   else
	      error("Could not read error message from Casio slave");
	else
	   error("Could not read reply from Casio slave");
	return(FALSE);
}

/************************************************************************/
EXPORT	int	create_casio_handler()

{	int	pid, commands[2], results[2];

	if (pipe(commands) == -1) {
	   fprintf(stderr, "cannot create a pipe: %s\n", sys_errlist[errno]);
	   exit(1);
	   }
	if (pipe(results) == -1) {
	   fprintf(stderr, "cannot create a pipe: %s\n", sys_errlist[errno]);
	   exit(1);
	   }
	if ((pid = fork()) == -1) {
	   fprintf(stderr, "cannot fork Casio handler: %s\n", sys_errlist[errno]);
	   exit(1);
	   }
	else if (pid == 0) {
	   close(commands[1]);
	   close(results[0]);
	   start_slave_handler(commands[0], results[1]);
	   exit(0);
	   }
	else {
	   close(commands[0]);
	   close(results[1]);
	   commands_out = fdopen(commands[1], "w");
	   results_in = fdopen(results[0], "r");
	   return(pid);
	   }
}

/************************************************************************/
EXPORT	destroy_casio_handler()

{
	close(commands_out);
	close(results_in);
}

/************************************************************************/
EXPORT	int	open_casio(port, baud, par)

char	*port;
int	baud;
int	par;

{
	buf[0] = OPEN_CASIO;
	buf[1] = baud;
	buf[2] = par;
	buf[3] = strlen(port);
	strcpy(buf + 4, port);
	if (do_write(buf, buf[3] + 4))
	   return(read_reply());
	else
	   return(FALSE);
}

/************************************************************************/
EXPORT	close_casio()

{
	buf[0] = CLOSE_CASIO;
	if (do_write(buf, 1))
	   read_reply();
}

/************************************************************************/
EXPORT	int	read_casio(buffer, len, max_time)

byte	*buffer;
int	len;
int	max_time;

{	int	amount;

	buf[0] = READ_CASIO;
	bcopy(&len, buf + 1, sizeof(int));
	bcopy(&max_time, buf + 5, sizeof(int));
	if (do_write(buf, 9))
	   if (read_reply())
	      if (fread(&amount, 1, sizeof(int), results_in) == sizeof(int))
	         if (fread(buffer, 1, amount, results_in) == amount)
	            return(amount);
	         else
	            error("Error reading %d bytes from Casio slave: %s", amount, sys_errlist[errno]);
	      else
	         error("Error reading block size from Casio slave: %s", sys_errlist[errno]);
	return(-1);
}

/************************************************************************/
EXPORT	int	write_casio(buffer, len)

byte	*buffer;
int	len;

{	int	result;

	buf[0] = WRITE_CASIO;
	bcopy(&len, buf + 1, sizeof(int));
	if (do_write(buf, 5))
	   if (do_write(buffer, len))
	      if (read_reply())
	         if (fread(&result, 1, sizeof(int), results_in) == sizeof(int))
	            return(result);
	         else
	            error("Could not read write reply from Casio slave: %s", sys_errlist[errno]);
	return(-1);
}

/************************************************************************/
EXPORT	enable_flow_control()

{
	buf[0] = ENABLE_FLOW_CONTROL;
	if (do_write(buf, 1))
	   return(read_reply());
	return(FALSE);
}

/************************************************************************/
EXPORT	disable_flow_control()

{
	buf[0] = DISABLE_FLOW_CONTROL;
	if (do_write(buf, 1))
	   return(read_reply());
	return(FALSE);
}

/************************************************************************/
EXPORT	int	flush_casio()

{
	buf[0] = FLUSH_CASIO;
	if (do_write(buf, 1))
	   return(read_reply());
	return(FALSE);
}

/************************************************************************/
EXPORT	unget_byte(b)

byte	b;

{
	buf[0] = UNGET_BYTE;
	buf[1] = b;
	if (do_write(buf, 2))
	   read_reply();
}

/************************************************************************/
EXPORT	wait_casio(delay)

int	delay;

{
	buf[0] = WAIT_CASIO;
	bcopy(&delay, buf + 1, sizeof(int));
	if (do_write(buf, 5))
	   read_reply();
}
